%% Blind Source Separation, Skript 1
% In diesem Skript wird eine vereinfachte Variante der Blind Source Separation 
% auf zwei generierte periodische Signale angewandt. Die STFT wird in diesem Fall 
% zunchst ausgelassen, die Anwendung der ICA findet im Zeitbereich der Signale 
% statt. Zu Evaluationszwecken wird die BSS zustzlich mittels einer PCA durchgefhrt.
%% Initialisierung
%%
clear all
close all
clc
run setupPath.m
%% Generate 2 periodic single frequency signals
%%
signal_Length = 1;   % Definieren Sie die Signallnge in Sekunden
sample_rate = 16000; % Definieren Sie die Abtastungen pro Sekunde in Hz

% Berechnen Sie einen Zeitvektor basierend auf der ...
%   Signallnge und der Abtastrate
time_vector = 0 : 1/sample_rate : signal_Length;

f1 = 7; % Definieren Sie die Frequenz von Quellsignal 1 in Hz
f2 = 3; % Definieren Sie die Frequenz von Quellsignal 2 in Hz

% Generieren Sie zwei periodische Quellsignale
sin_signal_f1 = sin(2*pi*f1*time_vector);
square_signal_f2 = sign(sin(2*pi*f2*time_vector));
square_signal_f2(~square_signal_f2) = 1;

source_signals = [sin_signal_f1; square_signal_f2];
%% Simulation der Signalmischung: Generieren Sie eine 2X2-Mischmatrix und berechnen Sie die resultierenden Mischsignale als Beobachtungsmatrix (#Beobachtungen X #Abtastungen)
%%
mix2 = [1.1 0.95; 1.2 0.85]; % Erstellen Sie eine Mischmatrix A


mixed_signals = mix2*source_signals; % Errechnen Sie die Beobachtungsmatrix
%% Komplettieren Sie das Skript centerRows.m in (./contributed/pca_ica)
%%
% Komplettieren Sie die Datei centerRows.m
[mixed_signals, mu] = centerRows(mixed_signals)
%% Complete the PCA (./contributed/pca_ica) and apply to mixtures
%%
% complete the algorithm by computing the truncated SVD 
recovered_signals_pca = PCA(mixed_signals, 2); % perform BSS using PCA
%% Evaluieren Sie die Performance der PCA unter Verwendung von Korrelationsmatrix und Plots
%%
correlation_matrix_pca = corr(source_signals', recovered_signals_pca');
% Errechnen Sie die Korrelation zwischen
% Quellsignal und wiederhergestelltem Signal
disp('Correlation Matrix:');
disp(correlation_matrix_pca);
figure
subplot(3,1,1);
hold on;
plot(time_vector, source_signals(1,:));
plot(time_vector, source_signals(2,:));
hold off;
title('Source Signals');
xlabel('Time (s)');
ylabel('Amplitude');
legend('s_1', 's_2');

subplot(3,1,2);
hold on;
plot(time_vector, mixed_signals(1,:));
plot(time_vector, mixed_signals(2,:));
hold off;
title('Mixed Signals');
xlabel('Time (s)');
ylabel('Amplitude');
legend('x_1', 'x_2');

subplot(3,1,3);
hold on;
plot(time_vector, recovered_signals_pca(1,:));
plot(time_vector, recovered_signals_pca(2,:));
hold off;
title('Recovered Signals (PCA)');
xlabel('Time (s)');
ylabel('Amplitude');
legend('y_1', 'y_2');
%% Komplettieren Sie das Skript whitenRows.m in (./contributed/pca_ica)
%%
% Komplettieren Sie die Datei whitenRows.m
%% Vervollstndigen Sie fastICA.m und wenden Sie den Algorithmus an mit sowohl Negentropie als auch Wlbung als Kostenfunktion (./contributed/pca_ica)
%%
% Vervollstndigen Sie den Algorithmus durch ...
%     Einfgen der Ableitungen der Kostenfunktionen

% Wenden Sie BSS mittels ICA mit Wlbung an
recovered_signals_ica_kurt = fastICA(mixed_signals, 2, 'kurtosis', 0);
% Wenden Sie BSS mittels ICA mit Negentropie an
recovered_signals_ica_neg = fastICA(mixed_signals, 2, 'negentropy', 0);
%% Evaulieren Sie die Performance von ICA unter Nutzung von Korrelationsmatrix und Plots
%%
% Verwenden Sie den corr-Befehl
correlation_matrix_ica_kurt = corr(source_signals', recovered_signals_ica_kurt');
% Verwenden Sie den corr-Befehl
correlation_matrix_ica_neg = corr(source_signals', recovered_signals_ica_neg');
disp('Correlation Matrix (Kurtosis):');
disp(correlation_matrix_ica_kurt);
disp('Correlation Matrix (Negentropy):');
disp(correlation_matrix_ica_neg);
figure
subplot(4,1,1);
hold on;
plot(time_vector, source_signals(1,:));
plot(time_vector, source_signals(2,:));
hold off;
title('Source Signals');
xlabel('Time (s)');
ylabel('Amplitude');
legend('s_1', 's_2');

subplot(4,1,2);
hold on;
plot(time_vector, mixed_signals(1,:));
plot(time_vector, mixed_signals(2,:));
hold off;
title('Mixed Signals');
xlabel('Time (s)');
ylabel('Amplitude');
legend('x_1', 'x_2');

subplot(4,1,3);
hold on;
plot(time_vector, recovered_signals_ica_kurt(1,:));
plot(time_vector, recovered_signals_ica_kurt(2,:));
hold off;
title('Recovered Signals (ICA kurtosis)');
xlabel('Time (s)');
ylabel('Amplitude');
legend('y_1', 'y_2');

subplot(4,1,4);
hold on;
plot(time_vector, recovered_signals_ica_neg(1,:));
plot(time_vector, recovered_signals_ica_neg(2,:));
hold off;
title('Recovered Signals (ICA negentropy)');
xlabel('Time (s)');
ylabel('Amplitude');
legend('y_1', 'y_2');